/*------------------------------------------------------------------
MPSDDoc.cpp: Implementation of the CMPSDDoc class
------------------------------------------------------------------*/
#include "stdafx.h"
#include "MPSDDoc.h"

IMPLEMENT_DYNCREATE(CMPSDDoc, CDocument)

CMPSDDoc::CMPSDDoc(): m_lTotalPages(0), m_lCurPage(0)
{
    m_bCurModified = FALSE;  m_bInsertPage = FALSE;
}

CMPSDDoc::~CMPSDDoc()
{
    ASSERT(m_lTotalPages ==m_aryPageData.GetSize());

    for (int i=0; i<m_lTotalPages; i++) {
        CPageData* pData =(CPageData*)m_aryPageData.GetAt(i);
        if (pData) delete pData;
    }
    m_aryPageData.RemoveAll();
}

BOOL CMPSDDoc::OnNewDocument()
{
    if (!CDocument::OnNewDocument()) return FALSE;

    m_aryPageData.Add( new CPageData );          
    m_lTotalPages =1;
    m_lCurPage    =0;
    return TRUE;
}

BOOL CMPSDDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
    if (m_bCurModified) SaveCurPage(); 
    return CDocument::OnSaveDocument(lpszPathName);
}

void CMPSDDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring()) 
    {                         // storing 
        ar << m_lTotalPages;    
        for (int i=0; i<m_lTotalPages; i++) {
           CPageData* pData =(CPageData*)m_aryPageData.GetAt(i);
           if (pData) pData->Serialize(ar);
        }  // for
    }      // if
    else 
    {                         // loading 
        ar >> m_lTotalPages;    
        for (int i=0; i<m_lTotalPages; i++) {
            CPageData* pData = new CPageData;
            if (pData) pData->Serialize(ar);
            m_aryPageData.Add(pData);          
        }  // for

        UpdateCurPage((CPageData*)m_aryPageData.GetAt(m_lCurPage));
    }      // else
}

void CMPSDDoc::PageAdd() 
{
    if (m_bCurModified) SaveCurPage(); 

    if (m_bInsertPage) {
        m_aryPageData.InsertAt(m_lCurPage, new CPageData);
        m_bInsertPage = FALSE;    // default for Add
    }
    else {
        m_aryPageData.Add(new CPageData);
        m_lCurPage = m_lTotalPages; // last is the current
    }

    m_lTotalPages++;
    ((CWnd*)m_viewList.GetHead())->SetWindowText(""); 
    SetModifiedFlag();
}

void CMPSDDoc::PageDelete() 
{
    CPageData* pData =(CPageData*)m_aryPageData.GetAt(m_lCurPage);
    if (pData) delete pData;
    m_aryPageData.RemoveAt(m_lCurPage);

    if (m_lCurPage == --m_lTotalPages) m_lCurPage--;
    UpdateCurPage((CPageData*)m_aryPageData.GetAt(m_lCurPage));
    SetModifiedFlag();
}

BOOL CMPSDDoc::PageImport(CString strFile)
{
    if (strFile.IsEmpty()) return FALSE;

    CPageData* pData = (CPageData*)m_aryPageData.GetAt(m_lCurPage);
    pData->m_iPageType = PDTYPE_TEXT;    

    if (!pData->m_strPageFile.IsEmpty()) {
        if (IDNO==AfxMessageBox("Overwrite the Current Page ?",
            MB_YESNO)) return TRUE;
    }
    else pData->m_strPageFile = GetTempFile();

    CopyFile(strFile, pData->m_strPageFile, FALSE); 
    UpdateCurPage(pData);
    SetModifiedFlag();
    return TRUE;
}

void CMPSDDoc::PageNotify(long lCurPage)
{
    if (m_bCurModified) SaveCurPage();

    m_lCurPage = lCurPage;
    UpdateCurPage((CPageData*)m_aryPageData.GetAt(m_lCurPage));
}

void CMPSDDoc::UpdateCurPage(CPageData* pData)
{
    if (pData->m_strPageFile.IsEmpty()) {
        ((CWnd*)m_viewList.GetHead())->SetWindowText(""); 
        return;
    }

    CFile f(pData->m_strPageFile, CFile::modeRead);
    pData->m_lPageLength = f.GetLength();

    CString str;
    LPTSTR p = str.GetBuffer(pData->m_lPageLength+1);
    p[pData->m_lPageLength] =0;

    f.Read(p, pData->m_lPageLength);
    f.Close();

    ((CWnd*)m_viewList.GetHead())->SetWindowText(p); 
    str.ReleaseBuffer( );
}

void CMPSDDoc::SaveCurPage()
{
    CString str;
    ((CWnd*)m_viewList.GetHead())->GetWindowText(str); 

    CPageData* pData = (CPageData*)m_aryPageData.GetAt(m_lCurPage);
    ASSERT(pData);

    pData->m_lPageLength = str.GetLength();
    if (pData->m_lPageLength==0) {
        if (!pData->m_strPageFile.IsEmpty()) {
            DeleteFile(pData->m_strPageFile);
            pData->m_strPageFile.Empty();
        }
        return;
    }

    LPTSTR p = str.GetBuffer(pData->m_lPageLength+1);
    p[pData->m_lPageLength] =0;

    if (pData->m_strPageFile.IsEmpty()) 
        pData->m_strPageFile = GetTempFile();

    CFile f(pData->m_strPageFile, // modeCreate to truncate, if any 
            CFile::modeCreate | CFile::modeWrite);
    f.Write(p, pData->m_lPageLength);
    f.Close();

    str.ReleaseBuffer( );
    m_bCurModified = FALSE;
}
