/*------------------------------------------------------------------
MPSDFrm.cpp: Implementation of the CMPSDFrame class
Author:      Zuoliu Ding
------------------------------------------------------------------*/
#include "stdafx.h"
#include "MPSDFrm.h"
#include "MPSDDoc.h"

IMPLEMENT_DYNCREATE(CMPSDFrame, CMDIChildWnd)
BEGIN_MESSAGE_MAP(CMPSDFrame, CMDIChildWnd)
    //{{AFX_MSG_MAP(CMPSDFrame)
    ON_COMMAND(ID_PAGE_ADD, OnPageAdd)
    ON_COMMAND(ID_PAGE_INSERT, OnPageInsert)
    ON_COMMAND(ID_PAGE_DELETE, OnPageDelete)
    ON_COMMAND(ID_PAGE_IMPORT, OnPageImport)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CMPSDFrame::OnCreateClient(LPCREATESTRUCT, 
                                CCreateContext* pCntxt) 
{
    if (!m_TabCtrl.Create(WS_VISIBLE |WS_CHILD |TCS_BOTTOM, 
        CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST)) return FALSE;

    m_TabCtrl.SetFont(CFont::FromHandle(
                      HFONT(::GetStockObject(DEFAULT_GUI_FONT))));

    CView* pView =(CView*)CreateView(pCntxt, AFX_IDW_PANE_FIRST+1);
    if (!pView) return FALSE;

    SetActiveView(pView);  // let it get focus
    m_pMPSDDoc = (CMPSDDoc*)GetActiveDocument();
    ASSERT(AfxIsValidAddress(m_pMPSDDoc, sizeof(CMPSDDoc)));

    return TRUE;
}

void CMPSDFrame::RecalcLayout(BOOL bNotify) 
{
    CMDIChildWnd::RecalcLayout(bNotify);

    CRect rect;
    m_TabCtrl.GetItemRect(0, rect);
    int iTabHeight = 5 +rect.Height();

    GetClientRect(rect);
    m_TabCtrl.MoveWindow(0, rect.bottom-iTabHeight, 
                            rect.Width(), iTabHeight);
  
    CView* pView =(CView*)GetActiveView();
    if (!pView) return;
    pView->MoveWindow(0, 0, rect.Width(), rect.bottom-iTabHeight);

}

BOOL CMPSDFrame::OnNotify(WPARAM, LPARAM lParam, LRESULT* pResult) 
{
    if (TCN_SELCHANGE == ((LPNMHDR)lParam)->code)
        m_pMPSDDoc->PageNotify(m_TabCtrl.GetCurSel());

    *pResult = 0;
    return TRUE;
}

void CMPSDFrame::OnPageAdd() 
{
    m_pMPSDDoc->PageAdd(); // add CPageData 

    TC_ITEM tci;
    tci.mask = TCIF_TEXT;
    CString str;
    str.Format("Page%d", m_pMPSDDoc->GetTotalNumber());                
    tci.pszText = str.GetBuffer(str.GetLength() +1);

    m_TabCtrl.InsertItem(m_pMPSDDoc->GetTotalNumber()-1, &tci);
    m_TabCtrl.SetCurSel(m_pMPSDDoc->GetCurPageIndex());
}

void CMPSDFrame::OnPageInsert() 
{
    m_pMPSDDoc->SetInsertPageFlag();
    OnPageAdd();
}

void CMPSDFrame::OnPageDelete() 
{
    if (1==m_pMPSDDoc->GetTotalNumber()) return; // only one page!
    if (IDNO==AfxMessageBox("Delete the Current Page ?", MB_YESNO))
        return;

    m_pMPSDDoc->PageDelete(); // delete cur CPageData 
    m_TabCtrl.DeleteItem(m_pMPSDDoc->GetTotalNumber());
    m_TabCtrl.SetCurSel(m_pMPSDDoc->GetCurPageIndex());
}

void CMPSDFrame::OnPageImport() 
{
    CFileDialog FDlg(TRUE, "TXT", "*.txt", OFN_FILEMUSTEXIST,
            "txt Files (*.txt)|*.txt| All Files (*.*)|*.*||", this);
    FDlg.m_ofn.lpstrTitle = "Import a Text File to This Page";  
    if (FDlg.DoModal() != IDOK)    return;   
    
    m_pMPSDDoc->PageImport(FDlg.GetPathName());
}

CMenu  CMPSDFrame::s_menuPage;

void CMPSDFrame::ActivateFrame(int nCmdShow) 
{                // called after CMPSDDoc::Serialize()
    TC_ITEM tci;
    tci.mask = TCIF_TEXT;
    CString str;
    m_TabCtrl.DeleteAllItems();

    for (int i=0; i<m_pMPSDDoc->GetTotalNumber(); i++) {
        str.Format("Page%d", i+1);                
        tci.pszText = str.GetBuffer(str.GetLength() +1);
        m_TabCtrl.InsertItem( i, &tci );
    }

    CMDIChildWnd::ActivateFrame(nCmdShow);

    CMenu* pMenu = AfxGetMainWnd()->GetMenu();
    if (!pMenu || !pMenu->GetMenuItemCount()) return;

    for (i = 0; i<(int)pMenu->GetMenuItemCount(); i++) {
        pMenu->GetMenuString(i, str, MF_BYPOSITION);
        if (str =="&Page") return; 
    }

    VERIFY(s_menuPage.CreatePopupMenu());
    s_menuPage.AppendMenu(MF_STRING,ID_PAGE_ADD, "Add Page");
    s_menuPage.AppendMenu(MF_STRING,ID_PAGE_INSERT, "Insert Page");
    s_menuPage.AppendMenu(MF_STRING,ID_PAGE_DELETE, "Delete Page");
    s_menuPage.AppendMenu(MF_STRING,ID_PAGE_IMPORT, "Import Page...");
    pMenu->InsertMenu(PAGE_MENU_POS, MF_POPUP |MF_BYPOSITION, 
                        (UINT)s_menuPage.m_hMenu, "&Page");
}


 