#define COBJMACROS
#define WIN32_LEAN_AND_MEAN
#pragma warning( disable: 4100 4201 4115 )

#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <assert.h>
#include <shldisp.h>
#include <shlguid.h>
#include "resource.h"
#include "enum.h"

#define dim( X ) ( sizeof( X ) / sizeof( ( X )[ 0 ] ) )

LPCOLESTR _awszWeekdays[] = {
    L"Monday", L"Tuesday" , L"Wednesday", L"Thursday",
    L"Friday", L"Saturday", L"Sunday"   ,
};

LPCOLESTR _awszConstants[] = {
    L"WM_NULL"         , L"WM_CREATE"      , L"WM_DESTROY"       ,
    L"WM_MOVE"         , L"WM_SIZE"        , L"WM_ACTIVATE"      ,
    L"WA_INACTIVE"     , L"WA_ACTIVE"      , L"WA_CLICKACTIVE"   ,
    L"WM_SETFOCUS"     , L"WM_KILLFOCUS"   , L"WM_ENABLE"        ,
    L"WM_SETREDRAW"    , L"WM_SETTEXT"     , L"WM_GETTEXT"       ,
    L"WM_GETTEXTLENGTH", L"WM_PAINT"       , L"WM_CLOSE"         ,
    L"WM_QUIT"         , L"WM_ERASEBKGND"  , L"WM_SYSCOLORCHANGE",
    L"WM_SHOWWINDOW"   , L"WM_WININICHANGE", L"WM_DEVMODECHANGE" ,
    L"WM_ACTIVATEAPP"  , L"WM_FONTCHANGE"  , L"WM_TIMECHANGE"    ,
    L"WM_CANCELMODE"   , L"WM_SETCURSOR"   , L"WM_MOUSEACTIVATE" ,
    L"WM_CHILDACTIVATE", L"WM_QUEUESYNC"   , L"WM_GETMINMAXINFO" ,
};

IAutoComplete2 *g_pAC1   = 0, *g_pAC2   = 0, *g_pAC3   = 0;
IEnumString    *g_pEnum1 = 0, *g_pEnum2 = 0, *g_pEnum3 = 0;

IEnumString *initAC( IAutoComplete2 *pAC, HWND hwndEdit,
    LPCOLESTR * pwszStrings, ULONG cNumStrings ) 
{
    if ( 0 != pAC ) {
        IEnumString *pStringProvider =
            createStringProvider( pwszStrings, cNumStrings );
        HRESULT hres = IAutoComplete2_Init(
            pAC, hwndEdit, (IUnknown *) pStringProvider, 0, 0 );
        IEnumString_Release( pStringProvider );
        assert( SUCCEEDED( hres ) );
        return pStringProvider;
    }
    return 0;
}

void onInitDialog( HWND hwnd ) {
    HWND hwndCombo = GetDlgItem( hwnd, IDC_HISTCONST );
    ComboBox_AddString( hwndCombo, _T( "WM_GETTEXTLENGTH" ) );
    ComboBox_AddString( hwndCombo, _T( "WM_PAINT"         ) );
    ComboBox_AddString( hwndCombo, _T( "WM_CLOSE"         ) );

    g_pEnum1 = initAC( g_pAC1, GetDlgItem( hwnd, IDC_WEEKDAYS ),
        _awszWeekdays, dim( _awszWeekdays ) );
    g_pEnum2 = initAC( g_pAC2, GetDlgItem( hwnd, IDC_CONSTANTS ),
        _awszConstants, dim( _awszConstants ) );
    g_pEnum3 = initAC( g_pAC3, GetTopWindow( hwndCombo ),
        _awszConstants, dim( _awszConstants ) );
}

void onCommand( HWND hwnd, int id, int codeNotify ) {
    if ( IDCANCEL == id && BN_CLICKED == codeNotify ) {
        EndDialog( hwnd, id );
    }
}

BOOL CALLBACK
dlgDemo( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam ) {
    switch ( msg ) {
    case WM_INITDIALOG:
        onInitDialog( hwnd );
        return TRUE;

    case WM_COMMAND:
        onCommand( hwnd, GET_WM_COMMAND_ID  ( wParam, lParam ),
                         GET_WM_COMMAND_CMD ( wParam, lParam ) );
        return TRUE;
    }
    return FALSE;   
}

IAutoComplete2 *createAC( DWORD dwFlags ) {
    IAutoComplete2 *pAutoComplete2 = 0;
    HRESULT hres = CoCreateInstance(
        &CLSID_AutoComplete, 0, CLSCTX_INPROC_SERVER,
        &IID_IAutoComplete2, (LPVOID *) &pAutoComplete2 );
    if ( SUCCEEDED( hres ) ) {
        assert( 0 != pAutoComplete2 );
        hres = IAutoComplete2_SetOptions( pAutoComplete2, dwFlags );
    }
    assert( SUCCEEDED( hres ) );
    return pAutoComplete2;
}

void displayProblem( void ) {
    #define showRefCount( interFace )                             \
        { TCHAR szOut[ 100 ]; IUnknown_AddRef( interFace );       \
        { const int i = IUnknown_Release( interFace );            \
        wsprintf( szOut, _T( #interFace ) _T( " = %d\r\n" ), i ); \
        } OutputDebugString( szOut ); }
    showRefCount( g_pEnum1 );
    showRefCount( g_pEnum2 );
    showRefCount( g_pEnum3 );
}

int APIENTRY
_tWinMain( HINSTANCE hinst, HINSTANCE h2, LPSTR psz, int nShow ) {
    CoInitialize( 0 );

    g_pAC1 = createAC( ACO_AUTOAPPEND );
    g_pAC2 = createAC( ACO_AUTOSUGGEST );
    g_pAC3 = createAC( ACO_AUTOSUGGEST | ACO_AUTOAPPEND );

    DialogBox( hinst, _T( "DemoDlg" ), HWND_DESKTOP, dlgDemo );

    displayProblem();

    if ( 0 != g_pAC1 ) {
        IAutoComplete2_Release( g_pAC1 );
        IAutoComplete2_Release( g_pAC2 );
        IAutoComplete2_Release( g_pAC3 );
    }

    CoUninitialize();

    return 0;
}
