#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include "enum.h"

typedef struct MyStringProvider {
    IEnumString _enumString; // This _must_ be first!
    ULONG _cRef;
    ULONG _iCurr;
    ULONG _cNumStrings;
    LPCOLESTR *_pwszStrings;
} MyStringProvider;

#define spThis ((MyStringProvider *) this)

STDMETHODIMP_( ULONG ) myAddRef( IEnumString *this ) {
    return ++spThis->_cRef;
}

STDMETHODIMP myQueryInterface( 
    IEnumString *this, REFIID refIID, LPVOID *pInterface )
{
    if ( IsEqualGUID( refIID, &IID_IUnknown ) ||
         IsEqualGUID( refIID, &IID_IEnumString ) ) 
    {
        *pInterface = this;
        myAddRef( this );
        return S_OK;
    }

    // Unsupported interface requested:
    *pInterface = 0;
    return E_NOINTERFACE;
}

STDMETHODIMP_( ULONG ) myRelease( IEnumString *this ) {
    assert( 0 < spThis->_cRef );
    if ( 0 < --spThis->_cRef ) {
        return spThis->_cRef;
    }
    free( this );
    return 0;
}

LPOLESTR my_strdup( LPCOLESTR pwsz ) {
    const UINT cch = wcslen( pwsz ) + 1;
    const UINT cb  = cch * sizeof( OLECHAR );
    LPOLESTR pwszRet = (LPOLESTR) CoTaskMemAlloc( cb );
    wcscpy( pwszRet, pwsz );
    assert( 0 == pwszRet[ cch - 1 ] );
    return pwszRet;
}

STDMETHODIMP myNext( IEnumString *this,
    ULONG celt, LPOLESTR *rgelt, ULONG *pceltFetched ) 
{
#if 0
    assert( !IsBadWritePtr( rgelt, celt * sizeof *rgelt ) );
    assert( 0 <= spThis->_iCurr && 
        spThis->_iCurr <= spThis->_cNumStrings );
#endif

    const ULONG nMaxElems = spThis->_cNumStrings - spThis->_iCurr;
    const ULONG ulFetched = __min( celt, nMaxElems );
    ULONG i = 0;
    for ( i = 0; i < ulFetched; ++i ) {
        rgelt[ i ] = my_strdup( 
            spThis->_pwszStrings[ spThis->_iCurr + i ] );
    }
    spThis->_iCurr += ulFetched;
    if ( 0 != *pceltFetched ) {
        *pceltFetched = ulFetched;
    }
    return ulFetched == celt ? S_OK : S_FALSE;
}

STDMETHODIMP mySkip( IEnumString *this, ULONG celt ) 
{
    assert( 0 <= spThis->_iCurr );
    assert( spThis->_iCurr <= spThis->_cNumStrings );
    spThis->_iCurr += celt;
    if ( spThis->_cNumStrings < spThis->_iCurr ) {
        spThis->_iCurr = spThis->_cNumStrings;
        return S_FALSE;
    }
    return S_OK;
}

STDMETHODIMP myReset( IEnumString *this ) {
    spThis->_iCurr = 0;
    return S_OK;
}

#pragma warning( disable: 4100 )

STDMETHODIMP myClone( IEnumString *this, IEnumString **ppEnum ) {
    return E_NOTIMPL;
}

IEnumString *
createStringProvider( LPCOLESTR *pwszStrings, ULONG cNumStrings ) {
    MyStringProvider *pMyStringProvider = 
        (MyStringProvider *) malloc( sizeof( MyStringProvider ) );
    if ( 0 != pMyStringProvider ) {
        static IEnumStringVtbl s_vtbl = {
            myQueryInterface, myAddRef, myRelease, 
            myNext, mySkip, myReset, myClone,
        };
        pMyStringProvider->_cRef = 1;
        pMyStringProvider->_iCurr = 0;
        pMyStringProvider->_cNumStrings = cNumStrings;
        pMyStringProvider->_pwszStrings = pwszStrings;
        pMyStringProvider->_enumString.lpVtbl = &s_vtbl;
    }
    return (IEnumString *) pMyStringProvider;
}
